;------------------------------------------------------------------------------
; File Name :      asr.asm
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.01
;      Date:      Aug 25, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------
; Software interrupt driven Asynchronous Serial Receiver
;------------------------------------------------------------------------------
           SUBTITLE        "Software interrupt driven Asynchronous Serial Receiver"
;------------------------------------------------------------------------------
#define __asr_asm
#include "pic.inc"
#include "vos.inc"
#include "asr.inc"
#include "asr_m.inc"

 constant ASR.cBitcount      = .8        ; values != 8 are not (yet) supported
 constant asr_cTMR1Prescaler = .1
 constant asr_cTMR1PSBits    = b'00'
 constant asr_cTMR1Period    = (ASR.Baudrate / .2 + _osc_clock / .4 / asr_cTMR1Prescaler) / ASR.Baudrate

 if asr_cTMR1Period < .400
    ; OS contributes interrupt latency approx 80 IC, 
    ; For reliable communiaction, 20% of period should be less than latency
    error Baudrate #v(ASR.Baudrate) is too high for clock #(_osc_clock)
 endif
;------------------------------------------------------------------------------
#define ASR.INTF    ASR.INTCON, INTF
#define ASR.INTE    ASR.INTCON, INTE
#define ASR.RX      ASR.Port, ASR.Pin
;------------------------------------------------------------------------------

 if ASR.cReceiver == ASR.eExternal
    extern ASR.Receiver
 else
 if ASR.cReceiver != ASR.eInternal
    constant ASR.Receiver = ASR.cReceiver
 endif
 endif
;------------------------------------------------------------------------------
; Instantiating encapsulated object, if requested
 if ASR.cReceiver == ASR.eInternal
VOS_Classes
ASR 
        VOS_mClass ASR_Handler, ASR_Init

VOS_Objects
ASR.Receiver 
        VOS_mPersistentObject ASR, VOS_cBCastID

        udata_ovr
asr.byteIn          res .1 
 endif
;------------------------------------------------------------------------------
ASR_ISRDATA udata
asr.data            res .1
asr.bitcount        res .1
;------------------------------------------------------------------------------
 errorlevel -.302
ASR     code
ASR_Init
        ; prepare Timer1 but do not start it
        _sfr_banksel T1CON
        movlw (0 << TMR1GE) | (asr_cTMR1PSBits << T1CKPS0) | (0 << T1OSCEN) | (0 << TMR1CS)
        movwf        T1CON
        ; Setup INT interrupt        
        asr.mSetEdge
        _sfr_banksel ASR.Tris
        bsf         ASR.Tris, ASR.Pin
        bcf         ASR.INTF
        bsf         ASR.INTE
        bsf         INTCON, PEIE            ; Enable periferial interrupt for TMR1
        return    
 if ASR.cReceiver == ASR.eInternal
ASR_Handler
        btfsc       VOS_bStateBCast
        return
        switch      INDF
        case ASR.cMsgIDGotByte, asr_gotByte
        return
asr_gotByte
        tstf        This
        skpnz
        return
        incf        FSR,F
        banksel     asr.byteIn
        movff       INDF, asr.byteIn
        VOS_mPostMsgIndir This, ASR.cMsgIDGotByte
        banksel     asr.byteIn
        movff       asr.byteIn, INDF
        return
 else
VOS_BootHandlers code                   ; If instance is not created ASR is initialised on boot
        call        ASR_Init
 endif

VOS_Interrupt0      code
        _sfr_banksel PIR1
        btfss       PIR1, TMR1IF
        bra         asr_notTMR1
        asr_mReadBit
        rrf         asr.data, F
        asr_mAddTMR1 -(asr_cTMR1Period)    ; 20 IC is latency time
        decfsz      asr.bitcount, F
        bra         asr_TMR1done
        ; Byte received, post a message
        VOS_mPostMessage ASR.Receiver, ASR.cMsgIDGotByte
        _sfr_banksel asr.data
        movff       asr.data, INDF          ; msg.data := asr.data
        ; re-enable INT interrupt
        bcf         ASR.INTF
        bsf         ASR.INTE
        ; disable TMR1 interrupt
        _sfr_banksel PIE1
        bcf         PIE1, TMR1IE
        _sfr_banksel T1CON
        bcf         T1CON, TMR1ON           ; stop timer
asr_TMR1done
        bcf         PIR1, TMR1IF
asr_notTMR1
        btfsc       ASR.INTE               ; 
        btfss       ASR.INTF               ; 
        bra         asr_notIINT
asr_dbgCheckPoint
        asr_mLoadTMR1 -(asr_cTMR1Period + asr_cTMR1Period / .2 - .17 - .32) ; 17 IC already expired, 2 IC HW latency, 32IC is half of possible SW latency
        bsf         T1CON, TMR1ON           ; start timer
        bcf         ASR.INTF
        bcf         ASR.INTE                ; disable INT interrupt
        bcf         PIR1, TMR1IF            ; clear TMR1IF
        _sfr_banksel  PIE1
        bsf         PIE1, TMR1IE            ; enable TMR1 interrupt
        banksel     asr.data
        clrf        asr.data                ; prepare working registers
        movlw       ASR.cBitcount
        movwf       asr.bitcount            
asr_notIINT

  end
