;------------------------------------------------------------------------------
; File Name :     main.asm
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.00
;      Date:      Dec 5, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
; VELOOS Example 3
;------------------------------------------------------------------------------

 include "pic.inc"
 include "vos.inc"
 include "asr.inc"
 include "vtm0.inc"
 include "main.inc"

 constant   Protocol.cMessageSize   = .3
 constant   Protocol.cCommandMask   = 0F8h
 constant   Protocol.cCommandID     = 048h      ; Direct message should have first byte in range 48..4C (H..L)
 constant   Protocol.cGroupCmdMask  = 0FEh
 constant   Protocol.cGroupCommand  = 04Eh      ; A group command, one of:
 constant   Protocol.cCommandAllOff = 04Eh      ; ALL OFF command ("N")
 constant   Protocol.cCommandAllOn  = 04Fh      ; ALL ON  command ("O")
 constant   Actuator.Port           = GPIO
 constant   Actuator.Tris           = TRISIO
 constant   Actuator.ON             = '1'
 constant   Actuator.OFF            = '0'
 constant   Actuator.ONPeriod       = .300
 constant   Actuator.ONPulses       = .1        ; just one high level
 constant   Actuator.OFFPeriod      = .100
 constant   Actuator.OFFPulses      = .5        ; three high level, two low level
 constant   Actuator.RAMSize        = 03h       ; byte state, word time
 constant   Generator.RAMSize       = 05h       ; byte state, word time, byte onperiod, byte offperiod
 constant   Generator.bLevel        = .0        ; bit 0 indicates next level to drive
 constant   Generator.bBusy         = .1        ; bit 1 indicates Generator is busy (not idle)
 global     ASR.Receiver
;------------------------------------------------------------------------------
	__CONFIG _WDT_ON & _INTRC_OSC_NOCLKOUT & _CP_OFF & _PWRTE_ON & _MCLRE_OFF
    errorlevel -.302
;------------------------------------------------------------------------------
; Declaring classes
VOS_Classes
ProtocolDriver
        VOS_mClass ProtocolDriver.Handler, ProtocolDriver.Init

Actuator
        VOS_mClass Actuator.Handler, Actuator.Init

Generator ; inherits Actuator.Init
        VOS_mClass Generator.Handler, Actuator.Init

;------------------------------------------------------------------------------
; Instantiating objects
VOS_Objects
ASR.Receiver
Commander       ; Commander is an istance of ProtocolDriver, also acts as ASR.Receiver
        VOS_mPersistentObject ProtocolDriver, ObjectMap
Generator1
        VOS_mPersistentObject Generator, Generator1_Data
Generator2
        VOS_mPersistentObject Generator, Generator2_Data
Actuator1
        VOS_mPersistentObject Actuator, Actuator1_Data
Actuator2
        VOS_mPersistentObject Actuator, Actuator2_Data
Actuator3
        VOS_mPersistentObject Actuator, Actuator3_Data

;------------------------------------------------------------------------------
; Object configuration data
DEEPROM     code
ObjectMap
        de low Generator1, low Generator2, low Actuator1, low Actuator2, low Actuator3
        de VOS_cBCastID, VOS_cBCastID, VOS_cBCastID ; stuffing MAP to have 8 bytes
Generator1_Data
        de Generator1_RAM, (1 << GP0)
Generator2_Data
        de Generator2_RAM, (1 << GP1)
Actuator1_Data
        de Actuator1_RAM,  (1 << GP4)
Actuator2_Data
        de Actuator2_RAM,  (1 << GP5)
Actuator3_Data
        de Actuator3_RAM,  (1 << GP0)           ; run out of outputs, sharing pin with Generator2
;------------------------------------------------------------------------------
; Object RAM allocation
            udata
Actuator1_RAM       res Actuator.RAMSize
Actuator2_RAM       res Actuator.RAMSize
Actuator3_RAM       res Actuator.RAMSize
Generator1_RAM      res Generator.RAMSize
Generator2_RAM      res Generator.RAMSize
ProtocolDriver_RAM
pd.state            res .1
pd.message          res Protocol.cMessageSize
            ;udata_shr    ; [!] uncomment on banked devices
 global Actuator.Latch
Actuator.Latch      res .1

pd_ovr      udata_ovr
pd.byte             res .1

act_ovr     udata_ovr
act_time            res .2
act_cmd             res .1

gen_ovr     udata_ovr
gen_cmd             res .2
;------------------------------------------------------------------------------
        code
; Protocol Driver Implementation
ProtocolDriver.Handler
        switch      INDF
        case        ASR.cMsgIDGotByte, pd_gotByte
        return
pd_gotByte
        ;banksel     pd.byte
        incf        FSR,F
        movff       INDF, pd.byte
        ;banksel     pd.state
        tstf        pd.state
        bnz         pd_saving
        movlw       Protocol.cCommandMask   ; first byte must be a valid command 
        andwf       pd.byte, W
        xorlw       Protocol.cCommandID
        skpz
        return                              ; command is not recognized, ignore byte
pd_saving
        movlw       low pd.message
        addwf       pd.state, W
        movwf       FSR
        movff       pd.byte, INDF
        incf        pd.state, F
        movlw       Protocol.cMessageSize
        xorwf       pd.state, W
        skpz
        return
pd_gotMessage
        clrf        pd.state    ; ready for a new message
        movlw       Protocol.cGroupCmdMask
        andwf       pd.message, W           ; W = pd.message & ~Protocol.cGroupCmdMask
        xorlw       Protocol.cGroupCommand  ; W = pd.message & ~Protocol.cGroupCmdMask
        skpz
        bra         pd_gotDirect
pd_gotGroup
        movlw       ~Protocol.cCommandMask                
        andwf       pd.message, W           ; W = object ID
        bra         pd_Post
pd_gotDirect
        movlw       ~Protocol.cCommandMask
        andwf       pd.message, W           ; W = object ID
        xorwf       pd.message, F           ; pd.message = Command (== 48)
pd_Post addwf       This, W                 ; This points to object map
        banksel     EEADR
        movwf       EEADR                   ; Read object handle
        bsf         EECON1, RD
        VOS_mPostMsgDyn EEDAT, pd.message   ; Post message to object, given in the message
        movff       pd.message + .1, INDF
        incf        FSR, F
        movff       pd.message + .2, INDF
        return
        
ProtocolDriver.Init
        ;banksel     pd.state
        clrf        pd.state
        return

;------------------------------------------------------------------------------
; Actuator Implementation
; class Actuator {
;   byte state;
;   word time; } 

Actuator.mReadMask macro
        banksel     EEADR
        incf        EEADR, F        ; read mask
        bsf         EECON1, RD
        movfw       EEDAT
 endm
;------------------------------------------------------------------------------
; protected methods

Actuator.doHigh
        Actuator.mReadMask
        ;banksel     Actuator.Latch
        iorwf       Actuator.Latch, F
        return

Actuator.doLow
        Actuator.mReadMask
        xorlw       0FFh
        ;banksel     Actuator.Latch
        andwf       Actuator.Latch, F
        return

Actuator.doTgl
        Actuator.mReadMask
        ;banksel     Actuator.Latch
        xorwf       Actuator.Latch, F
        return

Actuator.CompareTime
        incf        FSR, F
        movfw       INDF
        xorwf       act_time, W
        skpz
        return
        incf        FSR, F
        movfw       INDF
        xorwf       act_time+.1, W
        return

Actuator.LoadFSR
        movfw       This
        banksel     EEADR
        movwf       EEADR
        bsf         EECON1, RD
        movff       EEDAT, FSR
        return

; Actuator Message Handler 
Actuator.Handler
        switch      INDF
        case        VTMR_cMsgIDTimer1ms, ACT_1ms
        case        Protocol.cCommandID, ACT_gotCMD
        case        Protocol.cCommandAllOff, ACT_gotONOFF
        case        Protocol.cCommandAllOn, ACT_gotONOFF
        return
ACT_1ms ; Handling timer message
        INDF_movff  FSR, act_time, .2
        call        Actuator.LoadFSR
        tstf        INDF
        skpnz
        return      ; Actuator is idle
; Actuator State machine
act_doing
        call        Actuator.CompareTime
        skpz
        return
        call        Actuator.doTgl
        call        Actuator.LoadFSR                ; FSR = & state
        decfsz      INDF, F                         ; state --
        bra         act_done
        return
act_done
        incf        FSR, F
        INDF_mAddConstWord Actuator.OFFPeriod
        return

; Actuator Command Interpreter
ACT_gotCMD ; Handling Protocol command
        INDF_movff  FSR, act_cmd, .1
        call        Actuator.LoadFSR                ; FSR = & state
        tstf        INDF
        skpz 
        return      ; previous command is not complete
        switch      act_cmd
        case        Actuator.OFF, act_cmdON
        case        Actuator.ON, act_cmdOFF
        return
act_cmdON
        movlw       Actuator.ONPulses
        movwf       INDF
        incf        FSR,F
        INDF_mSetWordPlusConst VOS_Time, Actuator.ONPeriod
        goto        Actuator.doHigh       ; return Actuator.doHigh();
act_cmdOFF
        movlw       Actuator.OFFPulses
        movwf       INDF
        incf        FSR,F
        INDF_mSetWordPlusConst VOS_Time, Actuator.OFFPeriod
        goto        Actuator.doHigh     ; return Actuator.doHigh();
ACT_gotONOFF        
        movff       INDF, act_cmd
        call        Actuator.LoadFSR
        tstf        INDF
        skpz 
        return      ; previous command is not complete
        btfss       act_cmd,.0
        bra         act_cmdOFF
        bra         act_cmdON

; Actuator Initializer        
Actuator.Init
        call        Actuator.LoadFSR
        Actuator.mReadMask
        xorlw       0FFh
        banksel     Actuator.Port
        andwf       Actuator.Port, F
        banksel     Actuator.Tris
        andwf       Actuator.Tris, F
        clrf        INDF
        return

;------------------------------------------------------------------------------
; Generator Implementation
Generator.Handler
        switch      INDF
        case        VTMR_cMsgIDTimer1ms, GEN_1ms
        case        Protocol.cCommandID, GEN_gotCMD
        case        Protocol.cCommandAllOff, GEN_gotOFF
        case        Protocol.cCommandAllOn,  GEN_gotON
        return
; Generator State Machine
GEN_1ms
        INDF_movff  FSR, act_time, .2
        call        Actuator.LoadFSR
        tstf        INDF
        skpnz 
        return      ; generator is idle
gen_doing
        call        Actuator.CompareTime
        skpz
        return
        call        Actuator.LoadFSR        
        btfsc       INDF, Generator.bLevel
        bra         gen_hi
gen_lo  call        Actuator.doLow
        bsf         INDF, Generator.bLevel
        movlw       .4
        addwf       FSR, F                  ; FSR = & loperiod
        movfw       INDF                    ; W = loperiod
        decf        FSR, F
        bra         gen_nxt        
gen_hi  call        Actuator.doHigh
        bcf         INDF, Generator.bLevel
        movlw       .3
        addwf       FSR, F                  ; FSR = & hiperiod
        movfw       INDF                    ; W = hiperiod
gen_nxt decf        FSR,F                   ; 
        decf        FSR,F                   ; FSR = & time
        addwf       INDF, F                 ; time += W; // period
        skpc
        return
        incf        FSR, F
        incf        INDF, F
        return

; Generator Command Interpreter
GEN_gotCMD
        INDF_movff  FSR, gen_cmd, .2
        tstf        gen_cmd
        skpz
        tstf        gen_cmd + .1
        skpnz
        bra         gen_doOff
        call        Actuator.LoadFSR        ; FSR = & state
        movlw       (1<<Generator.bBusy)    ; Get Busy
        movwf       INDF            ; state = (1<<Generator.bBusy) | (idle ? (1<<Actuator.bFirst) : 0)
        incf        FSR, F                  ; FSR = & time
        movfw       VOS_Time
        addwf       gen_cmd, W
        movwf       INDF
        movfw       VOS_Time + .1           ; time = VOS_Time + gen_cmd
        skpnc
        addlw       .1
        incf        FSR, F
        movwf       INDF                    
        incf        FSR, F                  ; FSR = & onperiod
        movff       gen_cmd, INDF
        incf        FSR, F
        movff       gen_cmd + .1, INDF
        goto        Actuator.doHigh
GEN_gotON
        call        Actuator.LoadFSR        ; FSR = & state
        tstf        INDF
        skpz
        return                              ; already on
        movlw       .3
        addwf       FSR, F
        goto        GEN_gotCMD
GEN_gotOFF
gen_doOff
        call        Actuator.LoadFSR        ; FSR = & state
        clrf        INDF                    ; state = 0; // (idle)
        goto        Actuator.doLow
        return
;------------------------------------------------------------------------------

; Application initialization
VOS_BootHandlers code                           ; starts a section for the initialization routine
miha_init
        _pic_bankinit
        _pic_banksel OSCCAL
        call        _pic_maxrom             ;Get the cal value
        movwf       OSCCAL                  ;Calibrate
        _pic_banksel ANSEL
        clrf        ANSEL
        ;_pic_banksel OPTION_REG
        ;bcf         OPTION_REG, NOT_GPPU
        ;_pic_banksel WPU
        ;movlw       MASK
        ;movwf       WPU
        _pic_banksel CMCON
        movlw       b'111' << CM0           ; Turn off comparator
        movwf       CMCON
        clrf        Actuator.Latch
 end
