;------------------------------------------------------------------------------
; File Name :      vos.inc
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.04
;      Date:      Dec 03, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
; VELOOS - VEry Litlle Overhead Operating System
;------------------------------------------------------------------------------
; VELOOS interface - public primitives
;------------------------------------------------------------------------------
           SUBTITLE        "VELOOS interface - public primitives"           
;------------------------------------------------------------------------------
; VELOOS naming convention
; VOS - published entities
; vos - private entities
; ZZZ_Xxxx - public routine
; ZZZ_mXxx - public macro
; ZZZ_cXxx - public configuration constant/parametr
; ZZZ_eXxx - public enumeration constant
; ZZZ_bXxx - public bit field
;------------------------------------------------------------------------------
#define __vos_inc
#ifndef __pic_inc
    include "pic.inc"
#endif
    include "vos_cfg.inc"
;------------------------------------------------------------------------------
 ifdef __vos_asm
   implementation
 else
   interface
 endif
;------------------------------------------------------------------------------
 
 ; void * This;
 public This
 ; handle This_handle;
 public This_handle
 ; classref This_class
 public This_class

 ; struct VOSFlags * VOS_FLAGS;
 public VOS_FLAGS 
  #define VOS_bQueueEmpty       VOS_FLAGS, .0
  #define VOS_bQueueFull        VOS_FLAGS, .1
  #define VOS_bQueueOverflow    VOS_FLAGS, .2
  #define VOS_bTimeoutFlag      VOS_FLAGS, .3
  ;#define VOS_bStateIdle        VOS_FLAGS, .4
  #define VOS_bStateBCast       VOS_FLAGS, .5
  #define VOS_bSoftWDT          VOS_FLAGS, .7

;------------------------------------------------------------------------------
vos_extern macro entry
 ifndef __vos_asm
    extern entry
 endif
 endm

;------------------------------------------------------------------------------
vos_jump macro label
  if VOS_cDispScope == VOS_eDispNear 
    bra     label
  else
   if _pic_fam <= _pic16
    pagesel label
   endif
    goto    label
  endif
 endm

;------------------------------------------------------------------------------
vos_mDispEntry macro label
    vos_jump    label
 endm

;------------------------------------------------------------------------------
vos_mJumpToExit macro
    vos_extern  vos_HandlerExit
    if _pic_fam <= _pic16
      pagesel  vos_HandlerExit
    endif
    if VOS_cEnableBroadcast
      vos_extern  vos_BCastExit
      btfsc   VOS_bStateBCast
      goto    vos_BCastExit
    endif
    goto      vos_HandlerExit
 endm

;------------------------------------------------------------------------------
; This macro should be used to complete message handling routine
; All other interface entries should use return
VOS_mMsgExit macro
  if VOS_eInvoke == VOS_eInvokeCall
    return
  else
    vos_mJumpToExit
  endif
 endm

;------------------------------------------------------------------------------
; VOS_mClass 
; Defines a new class named ClassName
; Parameters:
;   fpHandler - a message handling routine:
;          bit (*)(message* FSR);
;   fpInit    - an initialization routine
; Prerequisites: must be declared in VOS_Classes code section
VOS_mClass macro fpHandler, fpInit
    vos_mDispEntry    fpHandler
    vos_mDispEntry    fpInit
 endm

; struct message {
;   handle  accepter;
;   byte    msg_id;
;   byte    data;
;   handle  sender; }

 if _pic_fam == _pic18
#define VOS_Objects VOS_ObjectSection code_pack
 else
#define VOS_Objects VOS_ObjectSection code
 endif

#define VOS_Classes VOS_ClassSection code

;------------------------------------------------------------------------------
; VOS_mPersistentObject
; Instantiates a persistent object
; Prerequisites: must be declared in VOS_Objects code section
VOS_mPersistentObject macro ClassRef, ptrObjectData
 if VOS_cPersistentLocation  == VOS_ePLEeprom
 de low(ClassRef)
 de low(ptrObjectData)
 endif
 if VOS_cPersistentLocation  == VOS_ePLRom
   ;ifdef EEPGD ; if EEPROM supports reading from ROM
   ; data low(ClassRef)
   ; data low(ptrObjectData)
   ;else
    retlw low(ClassRef)
    retlw low(ptrObjectData)
   ;endif
 endif
 endm

;------------------------------------------------------------------------------
 ifdef VOS_cEnableBroadcast
VOS_mBroadcastMsg macro msg_id
  VOS_mPostMessage VOS_cBCastID, msg_id
 endm
 endif

;------------------------------------------------------------------------------
; VOS_postMessage // void * VOS_postMessage(const handle obj_to,const msg_id);
; 
VOS_mPostMessage macro obj_to, msg_id
    vos_extern  vos_LoadPutPtr
    call        vos_LoadPutPtr          ; FSR = vos_putptr; ; (vos_putptr += msgSize) &= msgqueuesize
    movlw       low(obj_to)
    movwf       INDF
    movlw       msg_id
    vos_extern  vos_PostMsg
    call        vos_PostMsg
 endm

;------------------------------------------------------------------------------
; VOS_postMsgIndir // void * VOS_postMsgIndir(var obj_to,const msg_id);
VOS_mPostMsgIndir macro obj_ptr, msg_id
    vos_extern  vos_LoadPutPtr
    call        vos_LoadPutPtr          ; FSR = vos_putptr; ; (vos_putptr += msgSize) &= msgqueuesize
    if _pic_fam <= _pic16
    banksel     obj_ptr
    endif
    movff       obj_ptr, INDF
    movlw       msg_id
    vos_extern  vos_PostMsg
    call        vos_PostMsg
 endm

;------------------------------------------------------------------------------
; VOS_postMsgDyn // void * VOS_postMsgDyn(var obj_to,var msg_id);
VOS_mPostMsgDyn macro obj_ptr, msg_file
    vos_extern  vos_LoadPutPtr
    call        vos_LoadPutPtr          ; FSR = vos_putptr; ; (vos_putptr += msgSize) &= msgqueuesize
    if _pic_fam <= _pic16
    banksel     obj_ptr
    endif
    movff       obj_ptr, INDF
    banksel     msg_file
    movfw       msg_file
    vos_extern  vos_PostMsg
    call        vos_PostMsg
 endm


;------------------------------------------------------------------------------
vos_mLoadPCLA macro
    loadf PCLATH, scnstart_high VOS_ClassSection
    ifdef PCLATU
    loadf PCLATU, scnstart_upper VOS_ClassSection
    endif    
 endm
;------------------------------------------------------------------------------

VOS_cMessageSize         equ .4

; VOS System Messages
VTMR_cMsgIDTimer1s       equ 0F6h
VTMR_cMsgIDTimer1ms      equ 0F7h
VADC_cMsgIDAcquire       equ 0F8h
VADC_cMsgIDResultFull    equ 0F9h
VADC_cMsgIDResult8bit    equ 0FAh
vadc_cMsgIDACDDONE       equ 0FBh
VOS_cMsgIDQuerySleep     equ 0FDh
; This message is broadcast when message queue is empty
 if VOS_cEnableIdle
VOS_cMsgIDIdle           equ 0FEh
 endif

 ifndef __vos_kernel
#define clrwdt vos_noClrwdt
 endif
vos_noClrwdt macro
    messg Use of clrwdt is discouraged in VELOOS applications, no code generated
 endm
