;------------------------------------------------------------------------------
; File Name :      vtm0.asm
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.01
;      Date:      Dec 6, 2006
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------
; VELOOS Timer driver based on TMR0
; 
;------------------------------------------------------------------------------
; Revision history
; 1.01 - Published VOS.Time and VOS.TimeS

#define __vtm0_asm
#define __vos_kernel
 include "pic.inc"
 include "vos.inc"
 include "vtm0.inc"
 include "vtm0_m.inc"

; TMR0  persiod for 1 ms (@ 4 Mhz) is achived by setting prescaler to 4 and 
; TMR0 to -250, on TMR0 interrupt, TMR0 value will be reset
; Due to interrupt latency TMR0 value will (may) not be zero
; Thus, -250 shall be added to TMR0, this may lead to loosing prescaler value
;------------------------------------------------------------------------------
; Timer data
vtm0 udata
VOS_Time
vtmr_timems         res 2
 if VTMR_c1sTimer
VOS_TimeS
vtmr_times          res 2
vtmr_countms        res 2
 endif
vtm0_period         res 1

VOS_Classes
VTMR0 VOS_mClass VTM0_Handler, VTM0_Init

VOS_Objects
VTMR VOS_mPersistentObject VTMR0, TMR0

VTM0 code
  
VTM0_Init
    banksel     vtm0_period    
    movlw       vtm0_cTMR0
    movwf       vtm0_period
    _pic_bankinit
    clrwdt
    _pic_banksel OPTION_REG    
    movlw       ~((1<<T0CS)|(1<<PSA)|(1<<PS2)|(1<<PS1)|(1<<PS0))
    andwf       T0CON,W
    iorlw       vtm0_cT0CON
    movwf       T0CON
    _pic_banksel TMR0
    movlw       vtm0_cTMR0
    movwf       TMR0
    bcf         INTCON,T0IF
    bsf         INTCON,T0IE
    banksel vtmr_timems
    clrf    vtmr_timems
    clrf    vtmr_timems+1
    ifdef vtmr_countms
    loadf16 vtmr_countms, .1000 + .256  ; see explanation around vtm2_1ms
    endif
    ifdef   vtmr_times
    clrf    vtmr_times
    clrf    vtmr_times+1
    endif
    return

VTM0_Handler
    if VTMR_c1sTimer
    switch      INDF
    case        VTMR_cMsgIDTimer1ms, vtm0_1ms
    return
vtm0_1ms
    banksel     vtmr_countms
    decfsz      vtmr_countms, F
    return
    decfsz      vtmr_countms+1,F        ; this will end 256 ticks earlier
    return                              ; corrected by adding 256 to 1000
    loadf16 vtmr_countms, .1000 + .256  ; see explanation around vtm2_1ms
    infsnz      vtmr_times, F
    incf        vtmr_times+1, F
_1  VOS_mBroadcastMsg VTMR_cMsgIDTimer1s
    movff       vtmr_times, INDF
    incf        FSR, F
    movff       vtmr_times+1, INDF
    endif
    return           

VOS_Interrupt7 code
    btfss       INTCON, T0IF
    bra         vtm0_notTMR0
    VTMR_mHook
    banksel     vtmr_timems
    infsnz      vtmr_timems, F
    incf        vtmr_timems+1, F
    movfw       vtm0_period       
    _sfr_banksel TMR0
    addwf       TMR0,F
    VOS_mBroadcastMsg VTMR_cMsgIDTimer1ms
    banksel     vtmr_timems
    movff       vtmr_timems, INDF
    incf        FSR, F
    movff       vtmr_timems+1, INDF
    bcf         INTCON, T0IF
   if VOS_cEnableSoftWDT
    btfsc       VOS_bSoftWDT
    extern      vos_Timeout
    bra         vos_Timeout
    bsf         VOS_bSoftWDT
   endif
vtm0_notTMR0
 END
